"""
The MIT License (MIT)
Copyright (c) 2014-2015 University of Bristol

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
"""

import pymongo
import logging
from utils import Printable


class SensorBase(Printable):
    def __init__(self, server_config, db, collection):
        self.data = []
        self.sensors = set()
        self.collection = db[server_config["modalities"][collection]["collection"]]
        if collection == "annotations":
            self.fields = {}
            self.date_time_field = None
        else:
            self.fields = server_config["modalities"][collection]["fields"]
            self.date_time_field = server_config["modalities"][collection]["date_time_field"]

        self.experiments = {}
        self.docs = None

    def init_experiment(self, config):
        if self.date_time_field is not None:
            self.collection.ensure_index(self.date_time_field)

            # with CodeTimer('querying database ...'):
            self.docs = self.collection.find({self.date_time_field: {
                '$gte': config.experiment_start,
                '$lt': config.experiment_end
            }}, self.fields).sort(self.date_time_field, pymongo.ASCENDING)

            logging.info("Modality {0: <22}: {1} records found".format(self.collection.name, self.docs.count()))

    @property
    def sensors_short(self):
        try:
            return set(s.split('-')[1] for s in self.sensors)
        except ValueError:
            return self.sensors

    # def get_map_reduce_summary(self, frequency):
    #     docs = db.summaries.find({
    #         'value.modality': self.sensor,
    #         'value.frequency': frequency
    #     })
