"""
The MIT License (MIT)
Copyright (c) 2014-2015 University of Bristol

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
"""
from scipy.spatial import distance

from sensor_base import SensorBase
from code_timer import CodeTimer
from tqdm import tqdm


class Video(SensorBase):
    def __init__(self, server_config, db):
        self.event = server_config["modalities"]["video"]["event"]
        self.element_name = server_config["modalities"]["video"]["element_name"]

        super(Video, self).__init__(server_config, db, "video")

    def get_data(self):
        with CodeTimer('pulling video data from mongodb ...'):
            for doc in tqdm(self.docs, total=self.docs.count()):
                if self.event not in doc:
                    continue

                uid = doc['uid'].split('.')[0]
                row = {'datetime': doc[self.date_time_field], 'uid': uid}

                for element in doc[self.event]:
                    sensor = 'video-' + element[self.element_name]
                    self.sensors.add(sensor)
                    row[sensor] = element['v']

                if len(self.data) > 1 \
                        and row['datetime'] == self.data[-1]['datetime'] \
                        and row['video-frameID'] == self.data[-1]['video-frameID']:
                    # duplicate date found
                    d1 = distance.euclidean(row['video-3DCen'], self.data[-2]['video-3DCen'])
                    d2 = distance.euclidean(self.data[-1]['video-3DCen'], self.data[-2]['video-3DCen'])
                    if d1 < d2:
                        # new track is closer, replace last one. otherwise do nothing
                        self.data[-1] = row
                else:
                    self.data.append(row)

            # print(self.sensors)
            return self.data
