"""
The MIT License (MIT)
Copyright (c) 2014-2015 University of Bristol

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
"""
import numpy as np

from sensor_base import SensorBase
from code_timer import CodeTimer
from tqdm import tqdm


class Wearable(SensorBase):
    def __init__(self, server_config, db):
        self.event = server_config["modalities"]["wearable"]["event"]

        super(Wearable, self).__init__(server_config, db, "wearable")
        self.uids = {}

    def get_data(self):
        with CodeTimer('pulling wearable data from mongodb ...'):
            for doc in tqdm(self.docs, total=self.docs.count()):
                uid = doc['uid'] + '-' + doc['aid']
                if uid not in self.uids:
                    self.uids[uid] = str(len(self.uids))

                row = {'datetime': doc[self.date_time_field], 'uid': self.uids[uid]}

                for element in doc[self.event][0:2]:
                    if element['xl'] == '' or element['x'] == '' or element['y'] == '' or element['z'] == '':
                        continue
                    sensor = 'wearable-' + element['xl']
                    self.sensors.add(sensor)
                    row[sensor] = np.array([element['x'], element['y'], element['z']], dtype=float)

                    mag = 'wearable-mag-' + element['xl']
                    row[mag] = np.linalg.norm(row[sensor]) - 1
                    self.sensors.add(mag)

                if not all(row[sensor].size == 0 for sensor in self.sensors if sensor in row):
                    self.data.append(row)
                # else:
                #     print('Data empty for timestamp: {0}'.format(row['datetime']))

            # print(self.sensors)
            return self.data
